#!/usr/bin/env python3

from common import begin_server_test, end_server_test, run_transmitter, make_random_lines

def main():
    with begin_server_test(b"H") as control_sock:
        addrs = ["10.0.2.2:8091", "10.0.2.2:8092", "10.0.2.2:8093", "10.0.2.2:8094", "10.0.2.2:8095"]
        transmitter = run_transmitter(addrs, None)

        for i, line in enumerate(make_random_lines(1_000_000)):
            transmitter.stdin.write(line)
            transmitter.stdin.flush()
            if i == 0:
                assert control_sock.recv(1) == b">", "invalid control command from server"

        transmitter.stdin.close()
        return_code = transmitter.wait()

        assert return_code == 0, f"transmitter returned {return_code}"

        stderr_content = transmitter.stderr.read()

        assert stderr_content[-1:] == b"\n", "incomplete last line"

        stderr_lines = stderr_content[:-1].split(b"\n")
        assert len(stderr_lines) == 12, "wrong number of error lines"

        for addr in addrs[:3]:
            assert stderr_lines.count(bytes(f"{addr} - send error: Broken pipe", encoding='ascii')) == 1, \
                    f"wrong number or incorrect error messages for {addr}"
            assert stderr_lines.count(bytes(f"{addr} - connect error: Connection refused", encoding='ascii')) == 3, \
                    f"wrong number or incorrect error messages for {addr}"

        assert not transmitter.stdout.read(), f"wrong stdout content: {transmitter.stdout}"
        assert not transmitter.stderr.read(), f"wrong stderr content: {transmitter.stderr}"

        end_server_test(control_sock)

if __name__ == '__main__':
    main()
