#!/usr/bin/env python3

from common import begin_server_test, end_server_test, run_transmitter, make_random_lines

def main():
    with begin_server_test(b"G") as control_sock:
        transmitter = run_transmitter(
            ["10.0.2.2:8091", "10.0.2.2:8092", "10.0.2.2:8093", "10.0.2.2:8094", "10.0.2.2:8095"],
            None
        )
        for line in make_random_lines(100_000):
            transmitter.stdin.write(line)
            transmitter.stdin.flush()
            assert control_sock.recv(1) == b">", "invalid control command from server"

        transmitter.stdin.close()
        return_code = transmitter.wait()

        assert return_code == 0, f"transmitter returned {return_code}"

        assert not transmitter.stdout.read(), f"wrong stdout content: {transmitter.stdout}"
        assert not transmitter.stderr.read(), f"wrong stderr content: {transmitter.stderr}"

        end_server_test(control_sock)

if __name__ == '__main__':
    main()
