import requests
from hashlib import sha256
from pathlib import Path
import socket

INTERFACE = "enp0s8"

# You can change LARGE_FILE, MEDIUM_FILE and SMALL_FILE if the tests are too slow with your internet connection.
# Your solution will be tested with the default configuration of this file.

FILE_100_MB = {
    "url": "https://update.code.visualstudio.com/1.88.1/linux-deb-x64/stable",
    "checksum": "5e5716de6bf700f3a47bf344009328974da1d331a7b1260d884643eb808a4453",
}

FILE_20_MB = {
    "url": "https://github.com/rust-lang/rust/archive/refs/tags/1.77.2.tar.gz",
    "checksum": "d28370ce9b81e7022474501a8c76a945c849b40acf8665eab29a8212a4d687ae",
}

FILE_5_MB = {
    "url": "https://github.com/curl/curl/releases/download/curl-8_7_1/curl-8.7.1.tar.gz",
    "checksum": "f91249c87f68ea00cf27c44fdfa5a78423e41e71b7d408e5901a9896d905c495",
}

FILE_500_KB = {
    "url": "https://github.com/htop-dev/htop/archive/refs/tags/3.3.0.tar.gz",
    "checksum": "1e5cc328eee2bd1acff89f860e3179ea24b85df3ac483433f92a29977b14b045",
}

FILE_50_KB = {
    "url": "https://cdimage.debian.org/images/cloud/bookworm/20240415-1718/debian-12-nocloud-arm64-20240415-1718.json",
    "checksum": "b34b7fe538e41a46f4fa20e75a6ac51fe8ac3fa75540b8284b134022dae0e3ad",
}

FILE_500_B = {
    "url": "https://github.com/curl/curl/releases/download/curl-8_7_1/curl-8.7.1.tar.gz.asc",
    "checksum": "f2abc438269f3fe37749941d91069f4972d339cae9b289e536257d90098dcd1f",
}

LARGE_FILE = FILE_100_MB
MEDIUM_FILE = FILE_5_MB
SMALL_FILE = FILE_500_B

def check_download(file):
    r = requests.get(file["url"])
    assert sha256(r.content).hexdigest() == file["checksum"], "wrong checksum"

def get_stats():
    stats_path = Path(f'/sys/class/net/{INTERFACE}/statistics')
    stats = {}
    for stat in stats_path.iterdir():
        stats[stat.name] = int(stat.open().read())
    return stats

def check_stats_diff(stats_before, stats_after, stats_min_diff, stats_max_diff):
    for stat in stats_before:
        diff = stats_after[stat] - stats_before[stat]
        assert diff >= stats_min_diff.get(stat, 0), f"diff of {diff} for {stat} below threshold (check your values - maybe the threshold is too strict)"
        assert diff <= stats_max_diff.get(stat, 0), f"diff of {diff} for {stat} over threshold (check your values - maybe the threshold is too strict)"

def begin_server_test(test_id):
    control_sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    control_sock.connect(("10.0.2.2", 8090))
    control_sock.send(test_id)
    assert control_sock.recv(1) == b"+"
    return control_sock

def end_server_test(control_sock):
    assert control_sock.recv(1) == b"0"

def echo_test(test_data):
    with begin_server_test(b"A") as control_sock:
        with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as test_sock:
            test_sock.connect(("10.0.2.2", 8091))
            test_sock.sendall(test_data)
            test_sock.shutdown(socket.SHUT_WR)
            echo = b""
            while True:
                new_data = test_sock.recv(65536)
                if not new_data:
                    break
                echo += new_data

        assert echo == test_data

        end_server_test(control_sock)
